
//
//  WeakTimer.m
//  弱应用timer
//
//  Created by ly on 2017/4/28.
//  Copyright © 2017年 ZST. All rights reserved.
//

#import "WeakTimer.h"
#import <objc/runtime.h>
#import "NSObject+AfterDoing.h"
@interface WeakTimer ()

@property(nonatomic,assign) NSTimeInterval time ;
@property(nonatomic,weak) id target ;
@property(nonatomic,assign) SEL aSel ;
@property(nonatomic,weak) id userInfo ;
@property(nonatomic,assign) BOOL repeats ;
@property(nonatomic,copy) timerAction action ;
@end


@implementation WeakTimer

- (instancetype)initWithTimerWithTimeInterval:(NSTimeInterval)time target:(id)target selector:( SEL)aSel userInfo:(id)userInfo repeats:(BOOL)repeats
{
    self = [super init];
    if (self) {
        _time = time ;
        _target = target ;
        _aSel = aSel ;
        _userInfo = userInfo ;
        _repeats = repeats ;
        
    }
    return self;
}

- (instancetype)initWeakTimerWithTimeInterval:(NSTimeInterval)time repeats:(BOOL)shouldRepeats taeget:(id)target block:(timerAction)timerAction
{
    self = [super init];
    if (self) {
        _time = time ;
        _repeats = shouldRepeats ;
        _target = target ;
        _action = timerAction ;
    }
    return self;
}


+ (instancetype) addWeakTimerWithTimeInterval:(NSTimeInterval)time  repeats:(BOOL)shouldRepeats  taeget:(id)target block:(timerAction)timerAction  {

    WeakTimer * timer = [[WeakTimer alloc] initWeakTimerWithTimeInterval:time repeats:shouldRepeats taeget:target block:timerAction];
    
    if (shouldRepeats == NO) {
        
        [timer performBlockAfterDelay:time withDoBlock:timerAction];
        
    } else {
        
        
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            
            while (1) {

                
                [NSThread sleepForTimeInterval:time] ;
                if (timer.target == nil) {
                    break  ;
                }
                dispatch_async(dispatch_get_main_queue(), ^{
                    timer.action(timer);
                });

                

            }
            
        });
    }
    
    
    return timer ;
}



+ (instancetype) addTimerWithTimerWithTimeInterval:(NSTimeInterval)time target:(id)target selector:( SEL)aSel userInfo:(id)userInfo repeats:(BOOL)repeats {
    
    
    WeakTimer * weakTimer = [[WeakTimer alloc] initWithTimerWithTimeInterval:time target:target selector:aSel userInfo:userInfo repeats:repeats];
    
    if (repeats == NO) {
        [weakTimer.target performSelector:weakTimer.aSel withObject:userInfo afterDelay:time];
    } else {
        
        
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            
            while (1) {
                [NSThread sleepForTimeInterval:time] ;
                [weakTimer.target performSelectorOnMainThread:weakTimer.aSel withObject:userInfo waitUntilDone:NO ];
                if (weakTimer.target == nil) {
                    break ;
                }
            }
            
        });
        
        

        
    }
    
    
    return weakTimer ;
}

- (void)dealloc {
    
    NSLog(@"WeakTimer dealloc") ;
    
}




@end
